<?php
/******************************************************************************
* ManagePaid.php                                                              *
*******************************************************************************
* SMF: Simple Machines Forum                                                  *
* Open-Source Project Inspired by Zef Hemel (zef@zefhemel.com)                *
* =========================================================================== *
* Software Version:           SMF 1.1 RC2                                     *
* Software by:                Simple Machines (http://www.simplemachines.org) *
* Copyright 2001-2006 by:     Lewis Media (http://www.lewismedia.com)         *
* Support, News, Updates at:  http://www.simplemachines.org                   *
*******************************************************************************
* This program is free software; you may redistribute it and/or modify it     *
* under the terms of the provided license as published by Lewis Media.        *
*                                                                             *
* This program is distributed in the hope that it is and will be useful,      *
* but WITHOUT ANY WARRANTIES; without even any implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                        *
*                                                                             *
* See the "license.txt" file for details of the Simple Machines license.      *
* The latest version can always be found at http://www.simplemachines.org.    *
******************************************************************************/
if (!defined('SMF'))
	die('Hacking attempt...');

/*	This file contains all the screens that control settings for topics and
	posts.

	void ManagePaidSubscriptions()
		- the main entrance point for the 'Paid Subscription' screen.
		- accessed from ?action=paidsubscribe.
		- calls the right function based on the given sub-action.
		- defaults to sub-action 'view'.
		- requires admin_forum permission for admin based actions.

	void ModifySubscriptionSettings()
		- set any setting related to paid subscriptions.
		- requires the moderate_forum permission
		- accessed from ?action=paidsubscribe;sa=settings.

	void ViewSubscriptions()
		- view a list of all the current subscriptions
		- requires the admin_forum permission
		- accessed from ?action=paidsubscribe;sa=view.

	void ModifySubscription()
		- edit a subscription.
		- accessed from ?action=paidsubscribe;sa=modify.

	void ViewSubscribedUsers()
		- view a list of all the users who currently have a subscription.
		- requires the admin_forum permission.
		- subscription ID is required, in the form of $_GET['sid'].
		- accessed from ?action=paidsubscribe;sa=viewsub.

	void ModifyUserSubscription()
		- edit a users subscription.
		- accessed from ?action=paidsubscribe;sa=modifyuser.

	void reapplySubscriptions(array users)
		- reapplies all subscription rules for each of the users.

	void addSubscription(int id_subscribe, int id_member)
		- add/extend a subscription for a member.

	void removeSubscription(int id_subscribe, int id_member)
		- remove a subscription from a user.
*/

function ManagePaidSubscriptions()
{
	global $context, $txt, $scripturl, $sourcedir, $smfFunc;

	// Highlight the section.
	if (function_exists('adminIndex'))
		adminIndex('paid_subscriptions');
	elseif (empty($context['bar_area']))
	{
		require_once($sourcedir . '/Admin.php');
		AdminMain();
		$context['admin_area'] = 'paidsubscribe';
		$context['admin_section'] = 'members';
	}

	// Load the required language and template.
	loadLanguage('ManagePaid');
	loadTemplate('ManagePaid');

	$subActions = array(
		'settings' => array('ModifySubscriptionSettings', 'admin_forum'),
		'view' => array('ViewSubscriptions', 'admin_forum'),
		'viewsub' => array('ViewSubscribedUsers', 'admin_forum'),
		'modify' => array('ModifySubscription', 'admin_forum'),
		'modifyuser' => array('ModifyUserSubscription', 'admin_forum'),
	);

	// Default the sub-action to 'view subscriptions'.
	$_REQUEST['sa'] = isset($_REQUEST['sa']) && isset($subActions[$_REQUEST['sa']]) ? $_REQUEST['sa'] : 'view';

	// Make sure you can do this.
	isAllowedTo($subActions[$_REQUEST['sa']][1]);

	$context['page_title'] = $txt['paid_subscriptions'];

	// Tabs for browsing the different subscription functions.
	$context['admin_tabs'] = array(
		'title' => $txt['paid_subscriptions'],
		'help' => '',
		'description' => $txt['paid_subscriptions_desc'],
		'tabs' => array(
			array(
				'title' => $txt['paid_subs_view'],
				'description' => $txt['paid_subs_view_desc'],
				'href' => $scripturl . '?action=paidsubscribe;sa=view',
				'is_selected' => $_REQUEST['sa'] != 'settings',
			),
			array(
				'title' => $txt['paid_subs_settings'],
				'description' => $txt['paid_subs_settings_desc'],
				'href' => $scripturl . '?action=paidsubscribe;sa=settings',
				'is_selected' => $_REQUEST['sa'] == 'settings',
				'is_last' => true,
			),
		),
	);

	// Call the right function for this sub-acton.
	$subActions[$_REQUEST['sa']][0]();
}

// Modify which payment methods are to be used.
function ModifySubscriptionSettings()
{
	global $context, $txt, $db_prefix, $modSettings, $worldpay_pw, $twoco_pw, $authorize_tranid, $sourcedir, $smfFunc;

	// Setup the template.
	$context['sub_template'] = 'modify_settings';
	$context['page_title'] = $txt['paid_subs_settings'];

	$context['worldpay_password'] = empty($worldpay_pw) ? '' : 'grudgerocks';
	$context['authorize_transid'] = empty($authorize_tranid) ? '' : 'grudgerocks';
	$context['2co_secretword'] = empty($twoco_pw) ? '' : 'grudgerocks';
	$context['show_other_currency'] = !isset($modSettings['currency_code']) || isset($txt[$modSettings['currency_code'] . '_symbol']) ?  false : true;

	// Saving?
	if (isset($_POST['save']))
	{
		checkSession();

		// Pick the right currency.
		if ($_POST['currency'] == 'other')
		{
			$currencyCode = $_POST['currency_code'];
			$currencySymbol = $_POST['currency_symbol'];
		}
		else
		{
			$currencyCode = $_POST['currency'];
			$currencySymbol = $txt[$_POST['currency'] . '_symbol'];
		}

		// Update the actual settings.
		updateSettings(array(
			'paid_enabled' => empty($_POST['paid_enabled']) ? '0' : '1',
			'paid_email_to' => empty($_POST['paid_email_to']) ? '' : $_POST['paid_email_to'],
			'paid_email' => empty($_POST['paid_email']) ? '0' : (int) $_POST['paid_email'],
			'paidsubs_test' => empty($_POST['paidsubs_test']) ? '0' : '1',
			'paypal_email' => trim($_POST['paypal_email']) == '' ? '' : $_POST['paypal_email'],
			'worldpay_id' => trim($_POST['worldpay_id']) == '' ? '' : $_POST['worldpay_id'],
			'authorize_id' => trim($_POST['authorize_id']) == '' ? '' : $_POST['authorize_id'],
			'2co_id' => trim($_POST['2co_id']) == '' ? '' : $_POST['2co_id'],
			'nochex_email' => trim($_POST['nochex_email']) == '' ? '' : $_POST['nochex_email'],
			'currency_code' => $currencyCode,
			'currency_symbol' => $currencySymbol,
		));

		// Do the passwords that need to go into Settings.php
		require_once($sourcedir . '/Admin.php');

		if ($_POST['worldpay_password'] != 'grudgerocks' && ($_POST['worldpay_password'] != '' || !empty($worldpay_pw)))
			$config_vars['worldpay_pw'] = "'" . stripslashes($_POST['worldpay_password']) . "'";
		if ($_POST['authorize_transid'] != 'grudgerocks' && ($_POST['authorize_transid'] != '' || !empty($authorize_transid)))
			$config_vars['authorize_transid'] = "'" . stripslashes($_POST['authorize_transid']) . "'";
		if ($_POST['2co_secretword'] != 'grudgerocks' && ($_POST['2co_secretword'] != '' || !empty($twoco_pw)))
			$config_vars['twoco_pw'] = "'" . stripslashes($_POST['2co_secretword']) . "'";
		if (!empty($config_vars))
			updateSettingsFile($config_vars);

		redirectexit('action=paidsubscribe;sa=settings');
	}
}

// Are we looking at viewing the subscriptions?
function ViewSubscriptions()
{
	global $context, $txt, $db_prefix, $modSettings, $smfFunc;

	// Not made the settings yet?
	if (empty($modSettings['currency_symbol']))
		fatal_lang_error('paid_not_set_currency');

	// Setup the template.
	$context['sub_template'] = 'view_subscriptions';
	$context['page_title'] = $txt['paid_subs_view'];

	loadSubscriptions();
}

// Adding, editing and deleting subscriptions.
function ModifySubscription()
{
	global $context, $txt, $db_prefix, $modSettings, $smfFunc;

	$context['sub_id'] = isset($_REQUEST['sid']) ? (int) $_REQUEST['sid'] : 0;
	$context['action_type'] = $context['sub_id'] ? (isset($_REQUEST['delete']) ? 'delete' : 'edit') : 'add';

	// Setup the template.
	$context['sub_template'] = $context['action_type'] == 'delete' ? 'delete_subscription' : 'modify_subscription';
	$context['page_title'] = $txt['paid_' . $context['action_type'] . '_subscription'];

	// Delete it?
	if (isset($_POST['delete_confirm']) && isset($_REQUEST['delete']))
	{
		$smfFunc['db_query']('', "
			DELETE FROM {$db_prefix}subscriptions
			WHERE id_subscribe = $context[sub_id]
			LIMIT 1", __FILE__, __LINE__);

		redirectexit('action=paidsubscribe;view');
	}

	// Saving?
	if (isset($_POST['save']))
	{
		// Some cleaning...
		$isActive = isset($_POST['active']) ? 1 : 0;
		$isRepeatable = isset($_POST['repeatable']) ? 1 : 0;
		$allowpartial = isset($_POST['allow_partial']) ? 1 : 0;
		$reminder = isset($_POST['reminder']) ? (int) $_POST['reminder'] : 0;
		$emailComplete = strlen($_POST['emailcomplete']) > 10 ? trim($_POST['emailcomplete']) : '';

		// Is this a fixed one?
		if ($_POST['duration_type'] == 'fixed')
		{
			// Clean the span.
			$span = $_POST['span_value'] . $_POST['span_unit'];

			// Sort out the cost.
			$cost = array('fixed' => $_POST['cost']);
		}
		// Flexible is harder but more fun ;)
		else
		{
			$span = 'F';

			$cost = array(
				'day' => $_POST['cost_day'],
				'week' => $_POST['cost_week'],
				'month' => $_POST['cost_month'],
				'year' => $_POST['cost_year'],
			);
		}
		$cost = serialize($cost);

		// Yep, time to do additional groups.
		$addgroups = array();
		if (!empty($_POST['addgroup']))
			foreach ($_POST['addgroup'] as $id => $dummy)
				$addgroups[] = (int) $id;
		$addgroups = implode(',', $addgroups);

		// Is it new?!
		if ($context['action_type'] == 'add')
		{
			$smfFunc['db_query']('', "
				INSERT INTO {$db_prefix}subscriptions
					(name, description, active, length, cost, id_group, addgroups, repeatable, allowpartial, email_complete, reminder)
				VALUES
					('$_POST[name]', '$_POST[desc]', $isActive, '$span', '$cost', $_POST[prim_group], '$addgroups', $isRepeatable, $allowpartial, '$emailComplete', $reminder)", __FILE__, __LINE__);
		}
		// Otherwise must be editing.
		else
		{
			// Don't do groups if there are active members
			$request = $smfFunc['db_query']('', "
				SELECT COUNT(*)
				FROM {$db_prefix}log_subscribed
				WHERE id_subscribe = $context[sub_id]
					AND status = 2", __FILE__, __LINE__);
			list ($disableGroups) = $smfFunc['db_fetch_row']($request);
			$smfFunc['db_free_result']($request);

			$smfFunc['db_query']('', "
				UPDATE {$db_prefix}subscriptions
				SET name = '$_POST[name]', description = '$_POST[desc]', active = $isActive, length = '$span',
				cost = '$cost'" . ($disableGroups ? '' : ", id_group = $_POST[prim_group], addgroups = '$addgroups'") . ",
				repeatable = $isRepeatable, allowpartial = $allowpartial, email_complete = '$emailComplete', reminder = $reminder
				WHERE id_subscribe = $context[sub_id]", __FILE__, __LINE__);
		}

		redirectexit('action=paidsubscribe;view');
	}

	// Defaults.
	if ($context['action_type'] == 'add')
	{
		$context['sub'] = array(
			'name' => '',
			'desc' => '',
			'cost' => array(
				'fixed' => 0,
			),
			'span' => array(
				'value' => '',
				'unit' => 'D',
			),
			'prim_group' => 0,
			'add_groups' => array(),
			'active' => 1,
			'repeatable' => 1,
			'allow_partial' => 0,
			'duration' => 'fixed',
			'email_complete' => '',
			'reminder' => 0,
		);
	}
	// Otherwise load up all the details.
	else
	{
		$request = $smfFunc['db_query']('', "
			SELECT name, description, cost, length, id_group, addgroups, active, repeatable, allowpartial, email_complete, reminder
			FROM {$db_prefix}subscriptions
			WHERE id_subscribe = $context[sub_id]
			LIMIT 1", __FILE__, __LINE__);
		while ($row = $smfFunc['db_fetch_assoc']($request))
		{
			// Sort the date.
			preg_match('~(\d*)(\w)~', $row['length'], $match);
			if (isset($match[2]))
			{
				$span_value = $match[1];
				$span_unit = $match[2];
			}
			else
			{
				$span_value = 0;
				$span_unit = 'D';
			}

			// Is this a flexible one?
			if ($row['length'] == 'F')
				$isFlexible = true;
			else
				$isFlexible = false;

			$context['sub'] = array(
				'name' => $row['name'],
				'desc' => $row['description'],
				'cost' => @unserialize($row['cost']),
				'span' => array(
					'value' => $span_value,
					'unit' => $span_unit,
				),
				'prim_group' => $row['id_group'],
				'add_groups' => explode(',', $row['addgroups']),
				'active' => $row['active'],
				'repeatable' => $row['repeatable'],
				'allow_partial' => $row['allowpartial'],
				'duration' => $isFlexible ? 'flexible' : 'fixed',
				'email_complete' => htmlspecialchars($row['email_complete']),
				'reminder' => $row['reminder'],
			);
		}
		$smfFunc['db_free_result']($request);

		// Does this have members who are active?
		$request = $smfFunc['db_query']('', "
			SELECT COUNT(*)
			FROM {$db_prefix}log_subscribed
			WHERE id_subscribe = $context[sub_id]
				AND status = 2", __FILE__, __LINE__);
		list ($context['disable_groups']) = $smfFunc['db_fetch_row']($request);
		$smfFunc['db_free_result']($request);
	}

	// Load up all the groups.
	$request = $smfFunc['db_query']('', "
		SELECT id_group, group_name
		FROM {$db_prefix}membergroups
		WHERE id_group != 3
			AND min_posts = -1", __FILE__, __LINE__);
	$context['groups'] = array();
	while ($row = $smfFunc['db_fetch_assoc']($request))
		$context['groups'][$row['id_group']] = $row['group_name'];
	$smfFunc['db_free_result']($request);
}

// View all the users subscribed to a particular subscription!
function ViewSubscribedUsers()
{
	global $context, $txt, $db_prefix, $modSettings, $scripturl, $options, $smfFunc;

	// Setup the template.
	$context['sub_template'] = 'view_subscribed_users';
	$context['page_title'] = $txt['viewing_users_subscribed'];

	// ID of the subscription.
	$context['sub_id'] = (int) $_REQUEST['sid'];
	$context['start'] = isset($_REQUEST['start']) ? (int) $_REQUEST['start'] : 0;

	// Do the sorting!
	$sort_methods = array(
		'name' => 'name',
		'status' => 'status',
		'start' => 'starttime',
		'end' => 'endtime'
	);

	// Set up the importantant sorting variables... if they picked one...
	if (!isset($_GET['sort']) || !isset($sort_methods[$_GET['sort']]))
	{
		$_GET['sort'] = 'name';
		$descending = !empty($options['view_newest_first']);
	}
	// ... and if they didn't...
	else
		$descending = isset($_GET['desc']);

	$context['sort_by'] = $_GET['sort'];
	$_GET['sort'] = $sort_methods[$_GET['sort']];
	$context['sort_direction'] = $descending ? 'down' : 'up';

	$search = '';

	// Searching?
	if (isset($_POST['ssearch']) && !empty($_POST['sub_search']))
	{
		$search .= ' AND mem.real_name LIKE \'%' . addslashes($_POST['sub_search']) . '%\'';
	}

	$request = $smfFunc['db_query']('', "
		SELECT id_subscribe, name, description, cost, length, id_group, addgroups, active
		FROM {$db_prefix}subscriptions
		WHERE id_subscribe = $context[sub_id]", __FILE__, __LINE__);

	// Not found it?!
	if ($smfFunc['db_num_rows']($request) == 0)
		fatal_lang_error(1);

	// Do the context.
	$row = $smfFunc['db_fetch_assoc']($request);
	$context['subscription'] = array(
		'id' => $row['id_subscribe'],
		'name' => $row['name'],
		'desc' => $row['description'],
		'active' => $row['active'],
	);
	$smfFunc['db_free_result']($request);

	// Handy string.
	$txt['view_users_subscribed'] = sprintf($txt['view_users_subscribed'], $row['name']);

	// Get the total amount of users.
	$request = $smfFunc['db_query']('', "
		SELECT COUNT(*) AS total_subs
		FROM {$db_prefix}log_subscribed
		WHERE id_subscribe = $context[sub_id]", __FILE__, __LINE__);
	list ($context['total_subscribers']) = $smfFunc['db_fetch_row']($request);
	$smfFunc['db_free_result']($request);

	// Now create the page index.
	$context['page_index'] = constructPageIndex($scripturl . '?action=paidsubscribe;sa=viewsub;sid=' . $context['sub_id'] . ';sort=' . $context['sort_by'] . ($context['sort_direction'] == 'down' ? ';desc' : ''), $context['start'], $context['total_subscribers'], 25);

	// Now get the details of the current lot.
	$request = $smfFunc['db_query']('', "
		SELECT ls.id_sublog, ls.id_member, IFNULL(mem.real_name, '$txt[guest]') AS name, starttime, endtime, status
		FROM {$db_prefix}log_subscribed AS ls
			LEFT JOIN {$db_prefix}members AS mem ON (mem.id_member = ls.id_member)
		WHERE ls.id_subscribe = $context[sub_id] $search
		ORDER BY $_GET[sort] " . ($context['sort_direction'] == 'down' ? 'DESC' : 'ASC') . "
		LIMIT $context[start], 25", __FILE__, __LINE__);
	$context['sub_users'] = array();
	while ($row = $smfFunc['db_fetch_assoc']($request))
		$context['sub_users'][] = array(
			'id' => $row['id_sublog'],
			'ID_MEMBER' => $row['id_member'],
			'name' => $row['name'],
			'startDate' => timeformat($row['starttime'], false),
			'endDate' => timeformat($row['endtime'], false),
			'status' => $row['status'],
			'status_text' => $row['status'] == 0 ? $txt['paid_pending'] : ($row['status'] == 1 ? $txt['paid_finished'] : $txt['paid_active']),
		);
	$smfFunc['db_free_result']($request);

}

// Edit or add a user subscription.
function ModifyUserSubscription()
{
	global $context, $txt, $db_prefix, $modSettings, $smfFunc;

	$context['log_id'] = isset($_REQUEST['lid']) ? (int) $_REQUEST['lid'] : 0;
	$context['sub_id'] = isset($_REQUEST['sid']) ? (int) $_REQUEST['sid'] : 0;
	$context['action_type'] = $context['log_id'] ? 'edit' : 'add';

	// Setup the template.
	$context['sub_template'] = 'modify_user_subscription';
	$context['page_title'] = $txt[$context['action_type'] . '_subscriber'];

	// Searching?
	if (isset($_POST['ssearch']))
	{
		return ViewSubscribedUsers();
	}
	// Saving?
	elseif (isset($_REQUEST['save_sub']))
	{
		// Work out the dates...
		$starttime = mktime($_POST['hour'], $_POST['minute'], 0, $_POST['month'], $_POST['day'], $_POST['year']);
		$endtime = mktime($_POST['hourend'], $_POST['minuteend'], 0, $_POST['monthend'], $_POST['dayend'], $_POST['yearend']);

		// Status.
		$status = $_POST['status'];

		// Verify the subscription exists.
		$request = $smfFunc['db_query']('', "
			SELECT id_subscribe
			FROM {$db_prefix}subscriptions
			WHERE id_subscribe = $context[sub_id]", __FILE__, __LINE__);
		if ($smfFunc['db_num_rows']($request) == 0)
			fatal_lang_error(1);
		$smfFunc['db_free_result']($request);

		// New one?
		if (empty($context['log_id']))
		{
			// Find the user...
			$request = $smfFunc['db_query']('', "
				SELECT id_member, id_group
				FROM {$db_prefix}members
				WHERE real_name = '$_POST[name]'
				LIMIT 1", __FILE__, __LINE__);
			if ($smfFunc['db_num_rows']($request) == 0)
				fatal_lang_error('error_member_not_found');
			
			list ($ID_MEMBER, $ID_GROUP) = $smfFunc['db_fetch_row']($request);
			$smfFunc['db_free_result']($request);

			// Ensure the member doesn't already have a subscription!
			$request = $smfFunc['db_query']('', "
				SELECT id_subscribe
				FROM {$db_prefix}log_subscribed
				WHERE id_subscribe = $context[sub_id]
					AND id_member = $ID_MEMBER", __FILE__, __LINE__);
			if ($smfFunc['db_num_rows']($request) != 0)
				fatal_lang_error('member_already_subscribed');
			$smfFunc['db_free_result']($request);

			// Actually put the subscription in place.
			if ($status == 2)
				addSubscription($context['sub_id'], $ID_MEMBER, 0, $starttime, $endtime);
			else
			{
				$smfFunc['db_query']('', "
					INSERT INTO {$db_prefix}log_subscribed
						(id_subscribe, id_member, old_id_group, starttime, endtime, status)
					VALUES
						($context[sub_id], $ID_MEMBER, $ID_GROUP, $starttime, $endtime, $status)", __FILE__, __LINE__);
				updateNextCheck();
			}
		}
		// Updating.
		else
		{
			$request = $smfFunc['db_query']('', "
				SELECT id_member, status
				FROM {$db_prefix}log_subscribed
				WHERE id_sublog = $context[log_id]", __FILE__, __LINE__);
			if ($smfFunc['db_num_rows']($request) == 0)
				fatal_lang_error(1);

			list ($ID_MEMBER, $old_status) = $smfFunc['db_fetch_row']($request);
			$smfFunc['db_free_result']($request);

			// Pick the right permission stuff depending on what the status is changing from/to.
			if ($old_status == 2 && $status != 2)
				removeSubscription($context['sub_id'], $ID_MEMBER);
			elseif ($status == 2 && $old_status != 2)
			{
				addSubscription($context['sub_id'], $ID_MEMBER, 0, $starttime, $endtime);
				updateNextCheck();
			}
			else
			{
				$smfFunc['db_query']('', "
					UPDATE {$db_prefix}log_subscribed
					SET starttime = $starttime, endtime = $endtime, status = $status
					WHERE id_sublog = $context[log_id]", __FILE__, __LINE__);
				updateNextCheck();
			}
		}

		// Done - redirect...
		redirectexit('action=paidsubscribe;sa=viewsub;sid=' . $context['sub_id']);
	}
	// Deleting?
	elseif (isset($_REQUEST['delete']) || isset($_REQUEST['finished']))
	{
		// Do the actual deletes!
		if (!empty($_REQUEST['delsub']))
		{
			$toDelete = array();
			foreach ($_REQUEST['delsub'] as $id => $dummy)
				$toDelete[] = (int) $id;

			$request = $smfFunc['db_query']('', "
				SELECT id_subscribe, id_member
				FROM {$db_prefix}log_subscribed
				WHERE id_sublog IN (" . implode(',', $toDelete) . ")", __FILE__, __LINE__);
			while ($row = $smfFunc['db_fetch_assoc']($request))
				removeSubscription($row['id_subscribe'], $row['id_member'], isset($_REQUEST['delete']));
			$smfFunc['db_free_result']($request);
		}
		redirectexit('action=paidsubscribe;sa=viewsub;sid=' . $context['sub_id']);
	}

	// Default attributes.
	if ($context['action_type'] == 'add')
	{
		$context['sub'] = array(
			'id' => 0,
			'start' => array(
				'year' => (int) strftime('%Y', time()),
				'month' => (int) strftime('%m', time()),
				'day' => (int) strftime('%d', time()),
				'hour' => (int) strftime('%H', time()),
				'min' => (int) strftime('%M', time()) < 10 ? '0' . (int) strftime('%M', time()) : (int) strftime('%M', time()),
				'last_day' => 0,
			),
			'end' => array(
				'year' => (int) strftime('%Y', time()),
				'month' => (int) strftime('%m', time()),
				'day' => (int) strftime('%d', time()),
				'hour' => (int) strftime('%H', time()),
				'min' => (int) strftime('%M', time()) < 10 ? '0' . (int) strftime('%M', time()) : (int) strftime('%M', time()),
				'last_day' => 0,
			),
			'status' => 2,
		);
		$context['sub']['start']['last_day'] = (int) strftime('%d', mktime(0, 0, 0, $context['sub']['start']['month'] == 12 ? 1 : $context['sub']['start']['month'] + 1, 0, $context['sub']['start']['month'] == 12 ? $context['sub']['start']['year'] + 1 : $context['sub']['start']['year']));
		$context['sub']['end']['last_day'] = (int) strftime('%d', mktime(0, 0, 0, $context['sub']['end']['month'] == 12 ? 1 : $context['sub']['end']['month'] + 1, 0, $context['sub']['end']['month'] == 12 ? $context['sub']['end']['year'] + 1 : $context['sub']['end']['year']));

		if (isset($_GET['uid']))
		{
			$request = $smfFunc['db_query']('', "
				SELECT real_name
				FROM {$db_prefix}members
				WHERE id_member = " . (int) $_GET['uid'], __FILE__, __LINE__);
			list ($context['sub']['username']) = $smfFunc['db_fetch_row']($request);
			$smfFunc['db_free_result']($request);
		}
		else
			$context['sub']['username'] = '';
	}
	// Otherwise load the existing info.
	else
	{
		$request = $smfFunc['db_query']('', "
			SELECT id_sublog, id_subscribe, starttime, endtime, status
			FROM {$db_prefix}log_subscribed
			WHERE id_sublog = $context[log_id]
			LIMIT 1", __FILE__, __LINE__);
		if ($smfFunc['db_num_rows']($request) == 0)
			fatal_lang_error(1);
		$row = $smfFunc['db_fetch_assoc']($request);
		$smfFunc['db_free_result']($request);

		$context['sub_id'] = $row['id_subscribe'];
		$context['sub'] = array(
			'id' => 0,
			'start' => array(
				'year' => (int) strftime('%Y', $row['starttime']),
				'month' => (int) strftime('%m', $row['starttime']),
				'day' => (int) strftime('%d', $row['starttime']),
				'hour' => (int) strftime('%H', $row['starttime']),
				'min' => (int) strftime('%M', $row['starttime']) < 10 ? '0' . (int) strftime('%M', $row['starttime']) : (int) strftime('%M', $row['starttime']),
				'last_day' => 0,
			),
			'end' => array(
				'year' => (int) strftime('%Y', $row['endtime']),
				'month' => (int) strftime('%m', $row['endtime']),
				'day' => (int) strftime('%d', $row['endtime']),
				'hour' => (int) strftime('%H', $row['endtime']),
				'min' => (int) strftime('%M', $row['endtime']) < 10 ? '0' . (int) strftime('%M', $row['endtime']) : (int) strftime('%M', $row['endtime']),
				'last_day' => 0,
			),
			'status' => $row['status'],
		);
		$context['sub']['start']['last_day'] = (int) strftime('%d', mktime(0, 0, 0, $context['sub']['start']['month'] == 12 ? 1 : $context['sub']['start']['month'] + 1, 0, $context['sub']['start']['month'] == 12 ? $context['sub']['start']['year'] + 1 : $context['sub']['start']['year']));
		$context['sub']['end']['last_day'] = (int) strftime('%d', mktime(0, 0, 0, $context['sub']['end']['month'] == 12 ? 1 : $context['sub']['end']['month'] + 1, 0, $context['sub']['end']['month'] == 12 ? $context['sub']['end']['year'] + 1 : $context['sub']['end']['year']));
	}
}

// Re-apply subscription rules.
function reapplySubscriptions($users)
{
	global $db_prefix, $smfFunc;

	// Make it an array.
	if (!is_array($users))
		$users = array($users);

	// Get all the members current groups.
	$groups = array();
	$request = $smfFunc['db_query']('', "
		SELECT id_member, id_group, additional_groups
		FROM {$db_prefix}members
		WHERE id_member IN (" . implode(',', $users) . ")", __FILE__, __LINE__);
	while ($row = $smfFunc['db_fetch_assoc']($request))
	{
		$groups[$row['id_member']] = array(
			'primary' => $row['id_group'],
			'additional' => explode(',', $row['additional_groups']),
		);
	}
	$smfFunc['db_free_result']($request);

	$request = $smfFunc['db_query']('', "
		SELECT ls.id_member, ls.old_id_group, s.id_group, s.addgroups
		FROM ({$db_prefix}log_subscribed AS ls, {$db_prefix}subscriptions AS s)
		WHERE s.id_subscribe = ls.id_subscribe
			AND ls.id_member IN (" . implode(',', $users) . ")
			AND ls.endtime > " . time(), __FILE__, __LINE__);
	while ($row = $smfFunc['db_fetch_assoc']($request))
	{
		// Specific primary group?
		if ($row['id_group'] != 0)
		{
			// If this is changing - add the old one to the additional groups so it's not lost.
			if ($row['id_group'] != $groups[$row['id_member']]['primary'])
				$groups[$row['id_member']]['additional'][] = $groups[$row['id_member']]['primary'];
			$groups[$row['id_member']]['primary'] = $row['id_group'];
		}

		// Additional groups.
		if (!empty($row['addgroups']))
			$groups[$row['id_member']]['additional'] = array_merge($groups[$row['id_member']]['additional'], explode(',', $row['addgroups']));
	}
	$smfFunc['db_free_result']($request);

	// Update all the members.
	foreach ($groups as $id => $group)
	{
		$group['additional'] = array_unique($group['additional']);
		foreach ($group['additional'] as $key => $value)
			if (empty($value))
				unset($group['additional'][$key]);
		$addgroups = implode(',', $group['additional']);

		$smfFunc['db_query']('', "
			UPDATE {$db_prefix}members
			SET id_group = $group[primary], additional_groups = '$addgroups'
			WHERE id_member = $id
			LIMIT 1", __FILE__, __LINE__);
	}
}

// Add or extend a subscription of a user.
function addSubscription($ID_SUBSCRIBE, $ID_MEMBER, $renewel = 0, $forceStartTime = 0, $forceEndTime = 0)
{
	global $db_prefix, $context, $smfFunc;

	// Take the easy way out...
	loadSubscriptions();

	// Exists, yes?
	if (!isset($context['subscriptions'][$ID_SUBSCRIBE]))
		return;

	$curSub = $context['subscriptions'][$ID_SUBSCRIBE];

	// Grab the duration.
	$duration = $curSub['num_length'];

	// If this is a renewel change the duration to be correct.
	if (!empty($renewel))
	{
		switch ($renewel)
		{
			case 'D':
				$duration = 86400;
				break;
			case 'W':
				$duration = 604800;
				break;
			case 'M':
				$duration = 2629743;
				break;
			case 'Y':
				$duration = 31556926;
				break;
			default:
				break;
		}
	}

	// Firstly, see whether it exists, and is active. If so then this is meerly an extension.
	$request = $smfFunc['db_query']('', "
		SELECT id_sublog, endtime, starttime
		FROM {$db_prefix}log_subscribed
		WHERE id_subscribe = $ID_SUBSCRIBE
			AND id_member = $ID_MEMBER
			AND status = 2", __FILE__, __LINE__);
	if ($smfFunc['db_num_rows']($request) != 0)
	{
		list ($id_sublog, $endtime, $starttime) = $smfFunc['db_fetch_row']($request);

		// If this has already expired but is active, extension means the period from now.
		if ($endtime < time())
			$endtime = time();
		if ($starttime == 0)
			$starttime = time();

		// Work out the new expiry date.
		$endtime += $duration;

		if ($forceEndTime != 0)
			$endtime = $forceEndTime;

		// As everything else should be good, just update!
		$smfFunc['db_query']('', "
			UPDATE {$db_prefix}log_subscribed
			SET endtime = $endtime, starttime = $starttime
			WHERE id_sublog = $id_sublog", __FILE__, __LINE__);

		updateNextCheck();
		return;
	}
	$smfFunc['db_free_result']($request);

	// If we're here, that means we don't have an active subscription - that means we need to do some work!
	$request = $smfFunc['db_query']('', "
		SELECT m.id_group, m.additional_groups
		FROM {$db_prefix}members AS m
		WHERE m.id_member = $ID_MEMBER", __FILE__, __LINE__);
	// Just incase the member doesn't exist.
	if ($smfFunc['db_num_rows']($request) == 0)
		return;

	list ($old_id_group, $additional_groups) = $smfFunc['db_fetch_row']($request);
	$smfFunc['db_free_result']($request);

	// Prepare additional groups.
	$newAddGroups = explode(',', $curSub['add_groups']);
	$curAddGroups = explode(',', $additional_groups);

	$newAddGroups = array_merge($newAddGroups, $curAddGroups);

	// Simple, simple, simple - hopefully... id_group first.
	if ($curSub['prim_group'] != 0)
	{
		$ID_GROUP = $curSub['prim_group'];

		// Ensure their old priviledges are maintained.
		if ($old_id_group != 0)
			$newAddGroups[] = $old_id_group;
	}
	else
		$ID_GROUP = $old_id_group;

	// Yep, make sure it's unique, and no empties.
	foreach ($newAddGroups as $k => $v)
		if (empty($v))
			unset($newAddGroups[$k]);
	$newAddGroups = array_unique($newAddGroups);
	$newAddGroups = implode(',', $newAddGroups);

	// Store the new settings.
	$smfFunc['db_query']('', "
		UPDATE {$db_prefix}members
		SET id_group = $ID_GROUP, additional_groups = '$newAddGroups'
		WHERE id_member = $ID_MEMBER", __FILE__, __LINE__);

	// Now log the subscription - maybe we have a dorment subscription we can restore?
	$request = $smfFunc['db_query']('', "
		SELECT id_sublog, endtime, starttime
		FROM {$db_prefix}log_subscribed
		WHERE id_subscribe = $ID_SUBSCRIBE
			AND id_member = $ID_MEMBER", __FILE__, __LINE__);
	//!!! Don't really need to do this twice...
	if ($smfFunc['db_num_rows']($request) != 0)
	{
		list ($id_sublog, $endtime, $starttime) = $smfFunc['db_fetch_row']($request);

		// If this has already expired but is active, extension means the period from now.
		if ($endtime < time())
			$endtime = time();
		if ($starttime == 0)
			$starttime = time();

		// Work out the new expiry date.
		$endtime += $duration;

		if ($forceEndTime != 0)
			$endtime = $forceEndTime;

		// As everything else should be good, just update!
		$smfFunc['db_query']('', "
			UPDATE {$db_prefix}log_subscribed
			SET starttime = $starttime, endtime = $endtime, old_id_group = $old_id_group, status = 2,
				reminder_sent = 0
			WHERE id_sublog = $id_sublog", __FILE__, __LINE__);

		updateNextCheck();
		return;
	}
	$smfFunc['db_free_result']($request);

	// Otherwise a very simple insert.
	$endtime = time() + $duration;
	if ($forceEndTime != 0)
		$endtime = $forceEndTime;

	if ($forceStartTime == 0)
		$starttime = time();
	else
		$starttime = $forceStartTime;

	$smfFunc['db_query']('', "
		INSERT INTO {$db_prefix}log_subscribed
			(id_subscribe, id_member, old_id_group, starttime, endtime, status)
		VALUES
			($ID_SUBSCRIBE, $ID_MEMBER, $old_id_group, $starttime, $endtime, 2)", __FILE__, __LINE__);

	updateNextCheck();
}

// Removes a subscription from a user, as in removes the groups.
function removeSubscription($ID_SUBSCRIBE, $ID_MEMBER, $delete = false)
{
	global $db_prefix, $context, $smfFunc;

	loadSubscriptions();

	// Load the user core bits.
	$request = $smfFunc['db_query']('', "
		SELECT m.id_group, m.additional_groups
		FROM {$db_prefix}members AS m
		WHERE m.id_member = $ID_MEMBER", __FILE__, __LINE__);

	// Just incase of errors.
	if ($smfFunc['db_num_rows']($request) == 0)
	{
		$smfFunc['db_query']('', "
			DELETE FROM {$db_prefix}log_subscribed
			WHERE id_member = $ID_MEMBER", __FILE__, __LINE__);
		return;
	}
	list ($ID_GROUP, $additional_groups) = $smfFunc['db_fetch_row']($request);
	$smfFunc['db_free_result']($request);

	// Get all of the subscriptions for this user - it will be necessary!
	$request = $smfFunc['db_query']('', "
		SELECT ls.id_subscribe, ls.old_id_group
		FROM {$db_prefix}log_subscribed AS ls
		WHERE ls.id_member = $ID_MEMBER
			AND status = 2", __FILE__, __LINE__);

	// What if like, there isn't any?
	if ($smfFunc['db_num_rows']($request) == 0)
		return;

	// These variables will be handy, honest ;)
	$removals = array();
	$allowed = array();
	$old_id_group = 0;
	$new_id_group = -1;
	while ($row = $smfFunc['db_fetch_assoc']($request))
	{
		if (!isset($context['subscriptions'][$row['id_subscribe']]))
			continue;

		// The one we're removing?
		if ($row['id_subscribe'] == $ID_SUBSCRIBE)
		{
			$removals = explode(',', $context['subscriptions'][$row['id_subscribe']]['add_groups']);
			if ($context['subscriptions'][$row['id_subscribe']]['prim_group'] != 0)
				$removals[] = $context['subscriptions'][$row['id_subscribe']]['prim_group'];
			$old_id_group = $row['old_id_group'];
		}
		// Otherwise things we allow.
		else
		{
			$allowed = array_merge($allowed, explode(',', $context['subscriptions'][$row['id_subscribe']]['add_groups']));
			if ($context['subscriptions'][$row['id_subscribe']]['prim_group'] != 0)
			{
				$allowed[] = $context['subscriptions'][$row['id_subscribe']]['prim_group'];
				$new_id_group = $context['subscriptions'][$row['id_subscribe']]['prim_group'];
			}
		}
	}
	$smfFunc['db_free_result']($request);

	// Now, for everything we are removing check they defintely are not allowed it.
	$existingGroups = explode(',', $additional_groups);
	foreach ($existingGroups as $key => $group)
		if (empty($group) || (in_array($group, $removals) && !in_array($group, $allowed)))
			unset($existingGroups[$key]);

	// Finally, do something with the current primary group.
	if (in_array($ID_GROUP, $removals))
	{
		// If this primary group is actually allowed keep it.
		if (in_array($ID_GROUP, $allowed))
			$existingGroups[] = $ID_GROUP;

		// Either way, change the id_group back.
		if ($new_id_group < 1)
		{
			// If we revert to the old id-group we need to ensure it wasn't from a subscription.
			foreach ($context['subscriptions'] as $id => $group)
				// It was? Make them a regular member then!
				if ($group['prim_group'] == $old_id_group)
					$old_id_group = 0;

			$ID_GROUP = $old_id_group;
		}
		else
			$ID_GROUP = $new_id_group;
	}

	// Crazy stuff, we seem to have our groups fixed, just make them unique
	$existingGroups = array_unique($existingGroups);
	$existingGroups = implode(',', $existingGroups);

	// Update the member
	$smfFunc['db_query']('', "
		UPDATE {$db_prefix}members
		SET id_group = $ID_GROUP, additional_groups = '$existingGroups'
		WHERE id_member = $ID_MEMBER", __FILE__, __LINE__);

	// Disable the subscription.
	if (!$delete)
		$smfFunc['db_query']('', "
			UPDATE {$db_prefix}log_subscribed
			SET status = 1
			WHERE id_member = $ID_MEMBER
				AND id_subscribe = $ID_SUBSCRIBE", __FILE__, __LINE__);
	// Otherwise delete it!
	else
		$smfFunc['db_query']('', "
			DELETE FROM {$db_prefix}log_subscribed
			WHERE id_member = $ID_MEMBER
				AND id_subscribe = $ID_SUBSCRIBE", __FILE__, __LINE__);
}

// This just kind of caches all the subscription data.
function loadSubscriptions()
{
	global $context, $db_prefix, $txt, $modSettings, $smfFunc;

	if (!empty($context['subscriptions']))
		return;

	$request = $smfFunc['db_query']('', "
		SELECT id_subscribe, name, description, cost, length, id_group, addgroups, active
		FROM {$db_prefix}subscriptions", __FILE__, __LINE__);
	$context['subscriptions'] = array();
	while ($row = $smfFunc['db_fetch_assoc']($request))
	{
		// Pick a cost.
		$costs = @unserialize($row['cost']);

		if ($row['length'] != 'F' && !empty($modSettings['currency_symbol']) && !empty($costs['fixed']))
			$cost = sprintf($modSettings['currency_symbol'], $costs['fixed']);
		else
			$cost = '???';

		// Do the span.
		preg_match('~(\d*)(\w)~', $row['length'], $match);
		if (isset($match[2]))
		{
			$num_length = $match[1];
			$length = $match[1] . ' ';
			switch ($match[2])
			{
				case 'D':
					$length .= $txt['paid_mod_span_days'];
					$num_length *= 86400;
					break;
				case 'W':
					$length .= $txt['paid_mod_span_weeks'];
					$num_length *= 604800;
					break;
				case 'M':
					$length .= $txt['paid_mod_span_months'];
					$num_length *= 2629743;
					break;
				case 'Y':
					$length .= $txt['paid_mod_span_years'];
					$num_length *= 31556926;
					break;
			}
		}
		else
			$length = '??';

		$context['subscriptions'][$row['id_subscribe']] = array(
			'id' => $row['id_subscribe'],
			'name' => $row['name'],
			'desc' => $row['description'],
			'cost' => $cost,
			'real_cost' => $row['cost'],
			'length' => $length,
			'num_length' => $num_length,
			'real_length' => $row['length'],
			'pending' => 0,
			'finished' => 0,
			'total' => 0,
			'active' => $row['active'],
			'prim_group' => $row['id_group'],
			'add_groups' => $row['addgroups'],
			'flexible' => $row['length'] == 'F' ? true : false,
		);
	}
	$smfFunc['db_free_result']($request);

	// Do the counts.
	$request = $smfFunc['db_query']('', "
		SELECT COUNT(id_sublog) AS member_count, id_subscribe, status
		FROM {$db_prefix}log_subscribed
		GROUP BY id_subscribe, status", __FILE__, __LINE__);
	while ($row = $smfFunc['db_fetch_assoc']($request))
	{
		$ind = $row['status'] == 0 ? 'pending' : ($row['status'] == 1 ? 'finished' : 'total');

		if (isset($context['subscriptions'][$row['id_subscribe']]))
			$context['subscriptions'][$row['id_subscribe']][$ind] = $row['member_count'];
	}
	$smfFunc['db_free_result']($request);
}

function updateNextCheck()
{
	global $modSettings, $db_prefix, $smfFunc;

	$request = $smfFunc['db_query']('', "
		SELECT MIN(endtime)
		FROM {$db_prefix}log_subscribed
		WHERE status = 2", __FILE__, __LINE__);
	list ($nextExpire) = $smfFunc['db_fetch_row']($request);
	$smfFunc['db_free_result']($request);

	// If there's nothing then don't worry.
	if ($nextExpire == 0)
		$nextExpire = 2222222222;

	if (empty($modSettings['paidNextExpire']) || $nextExpire < $modSettings['paidNextExpire'])
		updateSettings(array('paidNextExpire' => $nextExpire));
}

// Function called to clean up expired subscriptions.
function autoCheckSubscriptions()
{
	global $db_prefix, $smfFunc;

	// Find all the ones to remove.
	$request = $smfFunc['db_query']('', "
		SELECT id_subscribe, id_member
		FROM {$db_prefix}log_subscribed
		WHERE status = 2
			AND endtime < " . time(), __FILE__, __LINE__);
	while ($row = $smfFunc['db_fetch_assoc']($request))
		removeSubscription($row['id_subscribe'], $row['id_member']);
	$smfFunc['db_free_result']($request);

	// Store the next expire time.
	updateNextCheck();
}

?>